@@ -4,6 +4,7 @@ from typing import TYPE_CHECKING, Dict, List, Optional, Tuple
 from attrs import asdict, define, field
 
 if TYPE_CHECKING:
+    from .hashfile.db import HashFileDB
     from .hashfile.hash_info import HashInfo
     from .hashfile.meta import Meta
     from .hashfile.obj import HashFile
@@ -15,7 +16,7 @@ DELETE = "delete"
 UNCHANGED = "unchanged"
 
 
-@define
+@define(hash=True, order=True)
 class TreeEntry:
     in_cache: bool = field(default=False, eq=False)
     key: Tuple[str, ...] = ()
@@ -26,7 +27,7 @@ class TreeEntry:
         return bool(self.oid)
 
 
-@define
+@define(hash=True, order=True)
 class Change:
     old: TreeEntry = field(factory=TreeEntry)
     new: TreeEntry = field(factory=TreeEntry)
@@ -70,8 +71,10 @@ class DiffResult:
 ROOT = ("",)
 
 
-def diff(
-    old: Optional["HashFile"], new: Optional["HashFile"], cache
+def diff(  # noqa: C901
+    old: Optional["HashFile"],
+    new: Optional["HashFile"],
+    cache: "HashFileDB",
 ) -> DiffResult:
     from .objects.tree import Tree
 
@@ -91,7 +94,10 @@ def diff(
     def _get(obj, key):
         if not obj or key == ROOT:
             return None, (obj.hash_info if obj else None)
-
+        if not isinstance(obj, Tree):
+            # obj is not a Tree and key is not a ROOT
+            # hence object does not exist for a given key
+            return None, None
         return obj.get(key, (None, None))
 
     def _in_cache(oid, cache):
@@ -124,10 +130,5 @@ def diff(
             ret.deleted.append(change)
         else:
             assert change.typ == UNCHANGED
-            if not change.new.in_cache and not (
-                change.new.oid and change.new.oid.isdir
-            ):
-                ret.modified.append(change)
-            else:
-                ret.unchanged.append(change)
+            ret.unchanged.append(change)
     return ret
